/*       MAPA                Ignacio  Merino  Lagunilla                       */

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "global.hpp"

//#define 	DEBUG


int mapa_mundo::crear(void)
{
 int i;
 register int j;

	maxx = planox;
	maxy = planoy;
	if ( (plano = new (unsigned char[maxx * maxy])) == NULL)
	{
		printf("\aERROR: no hay memoria suficiente para el mapa.\n");
		if (fichero == CIERTO) fprintf(accion , "ERROR: no hay memoria suficiente para el mapa.\n");
		return(-1);
	}
#ifdef DEBUG
	printf("Creo el mapa con maxx = %d maxy = %d\n" , maxx , maxy);
	if (fichero == CIERTO) fprintf(accion , "Creo el mapa con maxx = %d maxy = %d\n" , maxx , maxy);
#endif
	for(i = 0 ; i < maxy ; i++)
		for(j = 0 ; j < maxx ; j++)
			plano[i * maxx + j] = 0;
#ifdef DEBUG
	printf("Inicializo el mapa a ceros\n");
	if (fichero == CIERTO) fprintf(accion , "Inicializo el mapa a ceros\n");
#endif
	for(j = 0 ; j < maxx ; j++) plano[0 * maxx + j] = 255;
	for(j = 0 ; j < maxy ; j++) plano[j * maxx + 0] = 255;
	for(j = 0 ; j < maxy ; j++) plano[j * maxx + (maxx - 1)] = 255;
	for(j = 0 ; j < maxx ; j++) plano[(maxy - 1) * maxx + j] = 255;
#ifdef DEBUG
	printf("Dibujo los limites del mapa\n");
	if (fichero == CIERTO) fprintf(accion , "Dibujo los limites del mapa\n");
#endif
	return(0);
}

int mapa_mundo::escribir(char *fich , int copia)
{
 FILE *map;
 int i;
 register int j;
 static int num = 1;
 char nombre[50] , version[10];

	strcpy(nombre , fich);
	if(copia != 0)
	{
		sprintf(version , ".%d" , num);
		strcat(nombre , version);
		++num;
	}
	if ( (map = fopen(nombre , "wt")) == NULL)
	{
		printf("\aERROR: No puedo habrir el fichero '%s'.\n" , nombre);
		if (fichero == CIERTO) fprintf(accion , "ERROR: No puedo habrir el fichero '%s'.\n" , nombre);
		return(-1);
	}
#ifdef DEBUG
	printf("Creo el fichero '%s' para escribir el mapa\n" , nombre);
	if (fichero == CIERTO) fprintf(accion , "Creo el fichero '%s' para escribir el mapa\n" , nombre);
#endif
	for(i = 0 ; i < maxy ; i++)
	{
		for(j = 0 ; j < maxx ; j++)
			fprintf(map , "%3d" , plano[i * maxx + j]);
		fputc('\n' , map);
	}
#ifdef DEBUG
	printf("Escribo el mapa en el fichero '%s'\n" , nombre);
	if (fichero == CIERTO) fprintf(accion , "Escribo el mapa en el fichero '%s'\n" , nombre);
#endif
	fclose(map);
	return(0);
}

unsigned char mapa_mundo::di_punto(int x , int y)
{
	if(x < 0 || x >= maxx || y < 0 || y >= maxy)
	{
#ifdef DEBUG
		printf("\aERROR: Punto fuera de rango X = %d Y = %d.\n" , x , y);
		if (fichero == CIERTO) fprintf(accion , "ERROR: Punto fuera de rango X = %d Y = %d.\n" , x , y);
#endif
		return(255);
	}
	return(plano[y * maxx + x]);
}

int mapa_mundo::punto(int x , int y , unsigned char color)
{
	if(x < 0 || x >= maxx || y < 0 || y >= maxy)
	{
#ifdef DEBUG
		printf("\aERROR: Punto fuera de rango X = %d Y = %d en el mapa.\n" , x , y);
		if (fichero == CIERTO) fprintf(accion , "ERROR: Punto fuera de rango X = %d Y = %d en el mapa.\n" , x , y);
#endif
		return(-1);
	}
	plano[y * maxx + x] = color;
#ifdef DEBUG
	printf("Dibujo el punto x = %d y = %d color %d en el mapa\n" , x , y , color);
	if (fichero == CIERTO) fprintf(accion , "Dibujo el punto x = %d y = %d color %d en el mapa\n" , x , y , color);
#endif
	return(0);
}

int mapa_mundo::recta(int x1 , int y1 , int x2 , int y2 , unsigned char color)
{
 int d , dx , dy , aincr , bincr , incr , aux;
 register int x;
 register int y;

	if(x1 < 0 || x1 >= maxx || y1 < 0 || y1 >= maxy || x2 < 0 || x2 >= maxx || y2 < 0 || y2 >= maxy)
	{
#ifdef DEBUG
		printf("\aERROR: Recta fuera de rango X1 = %d Y1 = %d X2 = %d Y2 = %d color %d en el mapa.\n" , x1 , y1 , x2 , y2 , color);
		if (fichero == CIERTO) fprintf(accion , "ERROR: Recta fuera de rango X1 = %d Y1 = %d X2 = %d Y2 = %d color %d en el mapa.\n" , x1 , y1 , x2 , y2 , color);
#endif
		return(-1);
	}
	if(y1 == y2)
	{
		if(x1 > x2) { aux = x1; x1 = x2; x2 = aux; }
		for(x = x1 ; x <= x2 ; x++) plano[y1 * maxx + x] = color;
#ifdef DEBUG
		printf("Dibujo la linea horizontal Y = %d X1 = %d X2 = %d color %d en el mapa\n" , y1 , x1 , x2 , color);
		if (fichero == CIERTO) fprintf(accion , "Dibujo la linea horizontal Y = %d X1 = %d X2 = %d color %d en el mapa\n" , y1 , x1 , x2 , color);
#endif
		return(0);
	}
	if(x1 == x2)
	{
		if(y1 > y2) { aux = y1; y1 = y2; y2 = aux; }
		for(y = y1 ; y <= y2 ; y++) plano[y * maxx + x1] = color;
#ifdef DEBUG
		printf("Dibujo la linea vertical X = %d Y1 = %d Y2 = %d color %d en el mapa\n" , x1 , y1 , y2 , color);
		if (fichero == CIERTO) fprintf(accion , "Dibujo la linea vertical X = %d Y1 = %d Y2 = %d color %d en el mapa\n" , x1 , y1 , y2 , color);
#endif
		return(0);
	}
	if(abs(x2 - x1) < abs(y2 - y1))
	{
		if(y1 > y2)
		{
			aux = x1; x1 = x2; x2 = aux;
			aux = y1; y1 = y2; y2 = aux;
		}
		incr = (x2 > x1) ?  1 : -1;
		dy = y2 - y1;
		dx = abs(x2 - x1);
		d = 2 * dx - dy;
		aincr = 2 * (dx - dy);
		bincr = 2 * dx;
		x = x1; y = y1;
		plano[y * maxx + x] = color;
		for (y = y1 ; y <= y2 ; ++y )
		{
			if ( d >= 0 ) { x += incr; d += aincr; }
			else d += bincr;
			plano[y * maxx + x] = color;
		}
#ifdef DEBUG
		printf("Dibujo la linea X1 = %d Y1 = %d X2 = %d Y2 = %d color %d en el mapa por el eje Y\n" , x1 , y1 , x2 , y2 , color);
		if (fichero == CIERTO) fprintf(accion , "Dibujo la linea X1 = %d Y1 = %d X2 = %d Y2 = %d color %d en el mapa por el eje Y\n" , x1 , y1 , x2 , y2 , color);
#endif
	}
	else
	{
		if ( x1 > x2 )
		{
			aux = x1; x1 = x2; x2 = aux;
			aux = y1; y1 = y2; y2 = aux;
		}
		incr = ( y2 > y1 ) ? 1 : -1;
		dx = x2 - x1;
		dy = abs( y2 - y1 );
		d = 2 * dy - dx;
		aincr = 2 * (dy - dx);
		bincr = 2 * dy;
		x = x1; y = y1;
		plano[y * maxx + x] = color;
		for (x = x1 ; x <= x2 ; ++x )
		{
			if ( d >= 0 ) { y += incr; d += aincr; }
			else d += bincr;
			plano[y * maxx + x] = color;
		}
#ifdef DEBUG
		printf("Dibujo la linea X1 = %d Y1 = %d X2 = %d Y2 = %d color %d en el mapa por el eje X\n" , x1 , y1 , x2 , y2 , color);
		if (fichero == CIERTO) fprintf(accion , "Dibujo la linea X1 = %d Y1 = %d X2 = %d Y2 = %d color %d en el mapa por el eje X\n" , x1 , y1 , x2 , y2 , color);
#endif
	}
	return(0);
}

void mapa_mundo::poligono(int xc , int yc , float ang , int n , punto_pol *p , unsigned char color)
{
 int i;
 float x1 , y1 , x2 , y2 , an , a , b;

	an = angulo(ang + p[0].di_ang());
	a = x1 = xc + p[0].di_dis() * cos(an);
	b = y1 = yc - p[0].di_dis() * sin(an);
#ifdef DEBUG
	printf("Dibujo el poligono X = %d Y = %d angulo = %d puntos = %d color %d en el mapa\n" , xc , yc , grados(ang) , n , color);
	if (fichero == CIERTO) fprintf(accion , "Dibujo el poligono X = %d Y = %d angulo = %d puntos = %d color %d en el mapa\n" , xc , yc , grados(ang) , n , color);
#endif
	for(i = 1 ; i < n ; i++)
	{
		an = angulo(ang + p[i].di_ang());
		x2 = xc + p[i].di_dis() * cos(an);
		y2 = yc - p[i].di_dis() * sin(an);
		this->recta((int)ceil(x1) , (int)ceil(y1) , (int)ceil(x2) , (int)ceil(y2) , color);
		x1 = x2; y1 = y2;
	}
	this->recta((int)ceil(x1) , (int)ceil(y1) , (int)ceil(a) , (int)ceil(b) , color);
	this->rellenar(xc , yc , 1 , xc , xc , color);
#ifdef DEBUG
	printf("Relleno el poligono X = %d Y = %d angulo = %d puntos = %d color %d en el mapa\n" , xc , yc , grados(ang) , n , color);
	if (fichero == CIERTO) fprintf(accion , "Relleno el poligono X = %d Y = %d angulo = %d puntos = %d color %d en el mapa\n" , xc , yc , grados(ang) , n , color);
#endif
}

void mapa_mundo::circulo(int xc , int yc , int radio , unsigned char color)
{
 register int x = 0;
 register int y = radio;
 long a = radio , a2 = a * a , dosa2 = 2 * a2;
 long d = a2 - a2 * a + a2 / 4L , dx = 0 , dy = dosa2 * a;

	while(dx < dy)
	{
		plano[(yc + y) * maxx + xc + x] = color;
		plano[(yc + y) * maxx + xc - x] = color;
		plano[(yc - y) * maxx + xc + x] = color;
		plano[(yc - y) * maxx + xc - x] = color;
		if(d > 0L) { --y; dy-=dosa2; d-=dy; }
		++x; dx+=dosa2; d+=a2+dx;
	}
	d+=(-(dx + dy)) / 2L;
	while(y >= 0)
	{
		plano[(yc + y) * maxx + xc + x] = color;
		plano[(yc + y) * maxx + xc - x] = color;
		plano[(yc - y) * maxx + xc + x] = color;
		plano[(yc - y) * maxx + xc - x] = color;
		if(d < 0L) { ++x; dx+=dosa2; d+=dx; }
		--y; dy-=dosa2; d+=a2-dy;
	}
#ifdef DEBUG
	printf("Dibujo el circulo X = %d Y = %d radio = %d color %d en el mapa\n" , xc , yc , radio , color);
	if (fichero == CIERTO) fprintf(accion , "Dibujo el circulo X = %d Y = %d radio = %d color %d en el mapa\n" , xc , yc , radio , color);
#endif
	this->rellenar(xc , yc , 1 , xc , xc , color);
#ifdef DEBUG
	printf("Relleno el circulo X = %d Y = %d radio = %d color %d en el mapa\n" , xc , yc , radio , color);
	if (fichero == CIERTO) fprintf(accion , "Relleno el circulo X = %d Y = %d radio = %d color %d en el mapa\n" , xc , yc , radio , color);
#endif
}

int mapa_mundo::rellenar(int sx , int sy , int d , int axl , int axr , unsigned char color)
{
 int x , y = sy , v;
 int xl = sx;
 int xr = sx;

	do{ --xl; v = plano[y * maxx + xl]; } while(v != color); ++xl;
 	do{ ++xr; v = plano[y * maxx + xr]; } while(v != color); --xr;
	this->recta(xl , y , xr , y , color);
	for(x = xl ; x <= xr ; x++)
	{
		v = plano[(y + d) * maxx + x];
		if(v != color) x = rellenar(x , y + d , d , xl , xr , color);
	}
	for(x = xl ; x < axl ; x++)
	{
		v = plano[(y - d) * maxx + x];
		if(v != color) x = rellenar(x , y - d , -d , xl , xr , color);
	}
	for(x = axr ; x < xr ; x++)
	{
		v = plano[(y - d) * maxx + x];
		if(v != color) x = rellenar(x , y - d , -d , xl , xr , color);
	}
	return(xr);
}
